@@ -30,7 +30,7 @@
     waiting for some event to happen.
 
     Lwt threads are cooperative in the sense that switching to another
-    thread is awlays explicit (with {!wakeup} or {!wakeup_exn}). When a
+    thread is always explicit (with {!wakeup} or {!wakeup_exn}). When a
     thread is running, it executes as much as possible, and then
     returns (a value or an error) or sleeps.
 
@@ -78,7 +78,12 @@ val bind : 'a t -> ('a -> 'b t) -> 'b t
       Note that [bind] is also often used just for synchronization
       purpose: [t'] will not execute before [t] is terminated.
 
-      The result of a thread can be bound several times. *)
+      The result of a thread can be bound several times.
+
+      Note that [bind] will not propagate backtraces correctly.
+      See {{:http://ocsigen.org/lwt/manual/} the manual}
+      for how to enable backtraces.
+  *)
 
 val (>>=) : 'a t -> ('a -> 'b t) -> 'b t
   (** [t >>= f] is an alternative notation for [bind t f]. *)
@@ -116,6 +121,14 @@ val return_true : bool t
 val return_false : bool t
   (** [return_false = return false] *)
 
+val return_ok : 'a -> ('a, _) Result.result t
+  (** [return_ok x] is equivalent to [return (Ok x)].
+      @since 2.6.0 *)
+
+val return_error : 'e -> (_, 'e) Result.result t
+  (** [return_error x] is equivalent to [return (Error x)].
+      @since 2.6.0 *)
+
 (** {2 Thread storage} *)
 
 type 'a key
@@ -131,7 +144,10 @@ val get : 'a key -> 'a option
 
 val with_value : 'a key -> 'a option -> (unit -> 'b) -> 'b
   (** [with_value key value f] executes [f] with [value] associated to
-      [key]. [key] is restored to its previous value after [f] terminates. *)
+      [key]. [key] is restored to its previous value after [f] terminates.
+
+      This function should not be applied within threads created with
+      {!Lwt_preemptive.detach}. *)
 
 (** {2 Exceptions handling} *)
 
@@ -240,11 +256,17 @@ val async : (unit -> 'a t) -> unit
       terminates (for instance, because it is looping). *)
 
 val ignore_result : 'a t -> unit
-  (** [ignore_result t] is like [Pervasives.ignore t] except that:
+(** [ignore_result t] behaves as follows:
 
-      - if [t] already failed, it raises the exception now,
-      - if [t] is sleeping and fails later, the exception will be
-        given to {!async_exception_hook}. *)
+    - if [t] has completed with a result, [ignore_result t] does nothing,
+    - if [t] has completed with an exception, [ignore_result t] raises the
+      exception,
+    - if [t] has not completed, [ignore_result t] evaluates to [()] immediately,
+      but if [t] completes later with an exception, it will be given to
+      {!async_exception_hook}.
+
+    Note that this means [ignore_result t] does not wait for [t] to complete. If
+    you need to wait, use [t >>= fun _ -> (* ...after t... *)]. *)
 
 val async_exception_hook : (exn -> unit) ref
   (** Function called when a asynchronous exception is thrown.
@@ -253,7 +275,11 @@ val async_exception_hook : (exn -> unit) ref
       backtrace if available and to exit the program.
 
       The behavior is undefined if this function raise an
-      exception. *)
+      exception.
+
+      See {{:http://ocsigen.org/lwt/manual/} the manual}
+      for how to enable backtraces.
+  *)
 
 (** {2 Sleeping and resuming} *)
 
@@ -286,14 +312,22 @@ val wakeup_later_exn : 'a u -> exn -> unit
 val waiter_of_wakener : 'a u -> 'a t
   (** Returns the thread associated to a wakener. *)
 
-type +'a result
-  (** Either a value of type ['a], either an exception. *)
+type +'a result = ('a, exn) Result.result
+  (** Either a value of type ['a], either an exception.
+
+      This type is defined as [('a, exn) Result.result] since 2.6.0. *)
 
 val make_value : 'a -> 'a result
-  (** [value x] creates a result containing the value [x]. *)
+  [@@ocaml.deprecated
+    " Use Result.Ok, which is the same as Ok since OCaml 4.03."]
+  (** [value x] creates a result containing the value [x].
+      @deprecated Since 2.6.0. Use {!Result.Ok} *)
 
 val make_error : exn -> 'a result
-  (** [error e] creates a result containing the exception [e]. *)
+  [@@ocaml.deprecated
+    " Use Result.Error, which is the same as Error since OCaml 4.03."]
+  (** [error e] creates a result containing the exception [e].
+      @deprecated Since 2.6.0. Use {!Result.Error} *)
 
 val of_result : 'a result -> 'a t
   (** Returns a thread from a result. *)
@@ -483,3 +517,5 @@ val backtrace_bind : (exn -> exn) -> 'a t -> ('a -> 'b t) -> 'b t
 val backtrace_catch : (exn -> exn) -> (unit -> 'a t) -> (exn -> 'a t) -> 'a t
 val backtrace_try_bind : (exn -> exn) -> (unit -> 'a t) -> ('a -> 'b t) -> (exn -> 'b t) -> 'b t
 val backtrace_finalize : (exn -> exn) -> (unit -> 'a t) -> (unit -> unit t) -> 'a t
+
+val abandon_wakeups : unit -> unit
