@@ -27,7 +27,7 @@ type t = { mutable locked : bool; mutable waiters : unit Lwt.u Lwt_sequence.t  }
 
 let create () = { locked = false; waiters = Lwt_sequence.create () }
 
-let rec lock m =
+let lock m =
   if m.locked then
     Lwt.add_task_r m.waiters
   else begin
