@@ -99,10 +99,10 @@ module Make(Ord: OrderedType) : (S with type elt = Ord.t) =
       function
         [] -> raise Not_found
       | ts ->
-          let (Node (x, r, c), ts) = get_min ts in
+          let (Node (_, _, c), ts) = get_min ts in
           union (List.rev c) ts
 
     let rec size l =
-      let rec sizetree (Node (_,_,tl)) = 1 + size tl in
+      let sizetree (Node (_,_,tl)) = 1 + size tl in
       List.fold_left (fun s t -> s + sizetree t) 0 l
   end
