@@ -21,7 +21,11 @@
  *)
 
 (* Integration with the toplevel for people who do not use the
-   enhanced toplevel (the utop project). *)
+   enhanced toplevel (the utop project). This module is deprecated. *)
+
+[@@@ocaml.deprecated
+" Use utop. See
+   https://github.com/diml/utop"]
 
 open Lwt.Infix
 
