@@ -27,6 +27,10 @@
 
 #if defined(HAVE_LIBEV)
 
+#define CAML_NAME_SPACE
+
+#include <assert.h>
+
 #include <caml/alloc.h>
 #include <caml/fail.h>
 #include <caml/mlvalues.h>
@@ -36,18 +40,46 @@
 #include <caml/callback.h>
 #include <ev.h>
 
+/* +-----------------------------------------------------------------+
+   | Backend types                                                   |
+   +-----------------------------------------------------------------+ */
+enum {
+  val_EVBACKEND_DEFAULT,
+  val_EVBACKEND_SELECT,
+  val_EVBACKEND_POLL,
+  val_EVBACKEND_EPOLL,
+  val_EVBACKEND_KQUEUE,
+  val_EVBACKEND_DEVPOLL,
+  val_EVBACKEND_PORT
+};
+
+static unsigned int backend_val(value v)
+{
+  switch (Int_val(v))
+  {
+  case val_EVBACKEND_DEFAULT : return 0;
+  case val_EVBACKEND_SELECT  : return EVBACKEND_SELECT;
+  case val_EVBACKEND_POLL    : return EVBACKEND_POLL;
+  case val_EVBACKEND_EPOLL   : return EVBACKEND_EPOLL;
+  case val_EVBACKEND_KQUEUE  : return EVBACKEND_KQUEUE;
+  case val_EVBACKEND_DEVPOLL : return EVBACKEND_DEVPOLL;
+  case val_EVBACKEND_PORT    : return EVBACKEND_PORT;
+  default: assert(0);
+  }
+}
+
 /* +-----------------------------------------------------------------+
    | Loops                                                           |
    +-----------------------------------------------------------------+ */
 
 static int compare_loops(value a, value b)
 {
-  return (int)(Data_custom_val(a) - Data_custom_val(b));
+  return (int)((char*)Ev_loop_val(a) - (char*)Ev_loop_val(b));
 }
 
 static long hash_loop(value loop)
 {
-  return (long)Data_custom_val(loop);
+  return (long)Ev_loop_val(loop);
 }
 
 static struct custom_operations loop_ops = {
@@ -70,9 +102,9 @@ static void nop(struct ev_loop *loop)
 {
 }
 
-CAMLprim value lwt_libev_init(value Unit)
+CAMLprim value lwt_libev_init(value backend)
 {
-  struct ev_loop *loop = ev_loop_new(EVFLAG_FORKCHECK);
+  struct ev_loop *loop = ev_loop_new(EVFLAG_FORKCHECK | backend_val(backend));
   if (!loop) caml_failwith("lwt_libev_init");
   /* Remove the invoke_pending callback. */
   ev_set_invoke_pending_cb(loop, nop);
@@ -114,12 +146,12 @@ CAMLprim value lwt_libev_unloop(value loop)
 
 static int compare_watchers(value a, value b)
 {
-  return (int)(Data_custom_val(a) - Data_custom_val(b));
+  return (int)((char*)Ev_io_val(a) - (char*)Ev_io_val(b));
 }
 
 static long hash_watcher(value watcher)
 {
-  return (long)Data_custom_val(watcher);
+  return (long)Ev_io_val(watcher);
 }
 
 static struct custom_operations watcher_ops = {
