@@ -1,4 +1,4 @@
-/* Lightweight thread library for Objective Caml
+/* Lightweight thread library for OCaml
  * http://www.ocsigen.org/lwt
  * Header lwt_unix
  * Copyright (C) 2010 Jérémie Dimino
@@ -23,9 +23,12 @@
 #ifndef __LWT_UNIX_H
 #define __LWT_UNIX_H
 
+#define CAML_NAME_SPACE
+
 #include <lwt_config.h>
 #include <caml/mlvalues.h>
 #include <caml/unixsupport.h>
+#include <caml/socketaddr.h>
 
 /* The macro to get the file-descriptor from a value. */
 #if defined(LWT_ON_WINDOWS)
@@ -57,12 +60,31 @@ char *lwt_unix_strdup(char *string);
 /* Raise [Lwt_unix.Not_available]. */
 void lwt_unix_not_available(char const *feature) Noreturn;
 
+#define LWT_NOT_AVAILABLE1(prim) \
+  CAMLprim value lwt_ ## prim(value a1) \
+  { lwt_unix_not_available(#prim); }
+#define LWT_NOT_AVAILABLE2(prim) \
+  CAMLprim value lwt_ ## prim(value a1, value a2) \
+  { lwt_unix_not_available(#prim); }
+#define LWT_NOT_AVAILABLE3(prim) \
+  CAMLprim value lwt_ ## prim(value a1, value a2, value a3) \
+  { lwt_unix_not_available(#prim); }
+#define LWT_NOT_AVAILABLE4(prim) \
+  CAMLprim value lwt_ ## prim(value a1, value a2, value a3, value a4) \
+  { lwt_unix_not_available(#prim); }
+#define LWT_NOT_AVAILABLE5(prim) \
+  CAMLprim value lwt_ ## prim(value a1, value a2, value a3, value a4, value a5) \
+  { lwt_unix_not_available(#prim); }
+#define LWT_NOT_AVAILABLE6(prim) \
+  CAMLprim value lwt_ ## prim(value a1, value a2, value a3, value a4, value a5, value a6) \
+  { lwt_unix_not_available(#prim); }
+
 /* +-----------------------------------------------------------------+
    | Notifications                                                   |
    +-----------------------------------------------------------------+ */
 
 /* Sends a notification for the given id. */
-void lwt_unix_send_notification(int id);
+void lwt_unix_send_notification(intnat id);
 
 /* +-----------------------------------------------------------------+
    | Threading                                                       |
@@ -158,8 +180,8 @@ struct lwt_unix_job {
   struct lwt_unix_job *next;
 
   /* Id used to notify the main thread in case the job do not
-     terminate immediatly. */
-  int notification_id;
+     terminate immediately. */
+  intnat notification_id;
 
   /* The function to call to do the work.
 
@@ -278,17 +300,17 @@ void lwt_unix_free_job(lwt_unix_job job);
 /* Define not implement methods. Deprecated: it is for the old
    mechanism with three externals. */
 #define LWT_UNIX_JOB_NOT_IMPLEMENTED(name)      \
-  CAMLprim value lwt_unix_##name##_job()        \
+  CAMLprim value lwt_unix_##name##_job(value Unit)        \
   {                                             \
     caml_invalid_argument("not implemented");	\
   }                                             \
                                                 \
-  CAMLprim value lwt_unix_##name##_result()     \
+  CAMLprim value lwt_unix_##name##_result(value Unit)     \
   {                                             \
     caml_invalid_argument("not implemented");	\
   }                                             \
                                                 \
-  CAMLprim value lwt_unix_##name##_free()       \
+  CAMLprim value lwt_unix_##name##_free(value Unit)       \
   {                                             \
     caml_invalid_argument("not implemented");	\
   }
