@@ -533,6 +533,7 @@ LWT_NOT_AVAILABLE1(unix_open_job)
 LWT_NOT_AVAILABLE1(unix_stat_job)
 LWT_NOT_AVAILABLE1(unix_lstat_job)
 LWT_NOT_AVAILABLE1(unix_fstat_job)
+LWT_NOT_AVAILABLE3(unix_utimes_job)
 LWT_NOT_AVAILABLE1(unix_isatty_job)
 LWT_NOT_AVAILABLE1(unix_stat_64_job)
 LWT_NOT_AVAILABLE1(unix_lstat_64_job)
@@ -569,3 +570,11 @@ LWT_NOT_AVAILABLE1(unix_getpwuid_job)
 LWT_NOT_AVAILABLE1(unix_getgrgid_job)
 LWT_NOT_AVAILABLE4(unix_wait4)
 LWT_NOT_AVAILABLE5(unix_bytes_recvfrom)
+LWT_NOT_AVAILABLE1(unix_valid_dir)
+LWT_NOT_AVAILABLE1(unix_invalidate_dir)
+LWT_NOT_AVAILABLE3(unix_writev)
+LWT_NOT_AVAILABLE3(unix_writev_job)
+LWT_NOT_AVAILABLE1(unix_iov_max)
+LWT_NOT_AVAILABLE2(unix_bind_job)
+LWT_NOT_AVAILABLE3(unix_readv)
+LWT_NOT_AVAILABLE3(unix_readv_job)
