@@ -28,16 +28,27 @@ type t
 type suite
   (** Type of a suite of tests *)
 
-val test_direct : name : string -> run : (unit -> bool) -> t
+val test_direct : string -> ?only_if:(unit -> bool) -> (unit -> bool) -> t
   (** Defines a test. [run] must returns [true] if the test succeeded
-      and [false] otherwise. *)
+      and [false] otherwise. [only_if] is used to conditionally skip the
+      test. *)
 
-val test : name : string -> run : (unit -> bool Lwt.t) -> t
-  (** Defines a test which returns a thread. *)
+val test : string -> ?only_if:(unit -> bool) -> (unit -> bool Lwt.t) -> t
+  (** Like [test_direct], but defines a test which runs a thread. *)
 
-val suite : name : string -> tests : t list -> suite
+val suite : string -> t list -> suite
   (** Defines a suite of tests *)
 
-val run : name : string -> suites : suite list -> unit
+val run : string -> suite list -> unit
   (** Run all the given tests and exit the program with an exit code
       of [0] if all tests succeeded and with [1] otherwise. *)
+
+val temp_name : unit -> string
+(** Generates the name of a temporary file (or directory) in [_build/]. Note
+    that a file at the path may already exist. *)
+
+val temp_file : unit -> string
+(** Creates a temporary file in [_build/] and evaluates to its path. *)
+
+val temp_directory : unit -> string
+(** Creates a temporary directory in [build/] and evaluates to its path. *)
